/* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

/*
 Copyright (C) 2011 Klaus Spanderen

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the license for more details.
*/

/*! \file gpubicgstab.hpp
    \brief C++/Boost/QL interface for the cusp BiCGStab algorithm
*/

#ifndef quantlib_gpu_bicgstab_hpp
#define quantlib_gpu_bicgstab_hpp

#if !defined(QL_NO_UBLAS_SUPPORT)
#if defined(QL_PATCH_MSVC)
#pragma warning(push)
#pragma warning(disable:4180)
#endif

#include <boost/numeric/ublas/matrix_sparse.hpp>

#if defined(QL_PATCH_MSVC)
#pragma warning(pop)
#endif

#include <ql/math/array.hpp>
#include <boost/scoped_ptr.hpp>

namespace QuantLib {
    class GPUBiCGStab {
      public:
        typedef boost::numeric::ublas::compressed_matrix<Real> matrix_type;

        GPUBiCGStab(const matrix_type& matrix);
        ~GPUBiCGStab();

        Disposable<Array> solve(const Array& b,
                                Size maxIter = 100, Real eps = 1e-7);

      private:
        class pimpl;
        boost::scoped_ptr<pimpl> pImpl_;
    };
}

#endif
#endif
